@@ -26,9 +26,9 @@
 
 #include <telepathy-glib/base-connection.h>
 #include <telepathy-glib/presence-mixin.h>
-#include <wocky/wocky-session.h>
+#include <wocky/wocky.h>
 
-#include <gabble/connection.h>
+#include <gabble/plugin-connection.h>
 #include <gabble/sidecar.h>
 #include <gabble/types.h>
 
@@ -48,11 +48,23 @@ typedef struct _GabblePluginInterface GabblePluginInterface;
 typedef void (*GabblePluginCreateSidecarImpl) (
     GabblePlugin *plugin,
     const gchar *sidecar_interface,
-    GabbleConnection *connection,
+    GabblePluginConnection *plugin_connection,
     WockySession *session,
     GAsyncReadyCallback callback,
     gpointer user_data);
 
+/* The return type should be a new GPtrArray* which will be freed
+ * straight after this function is called, so the pointer array must
+ * not have a free function. */
+typedef GPtrArray * (*GabblePluginCreateChannelManagersImpl) (
+    GabblePlugin *plugin,
+    GabblePluginConnection *plugin_connection);
+
+typedef GabbleSidecar * (*GabblePluginCreateSidecarFinishImpl) (
+     GabblePlugin *plugin,
+     GAsyncResult *result,
+     GError **error);
+
 struct _GabblePluginPrivacyListMap {
     const gchar *presence_status_name;
     const gchar *privacy_list_name;
@@ -74,9 +86,14 @@ struct _GabblePluginInterface {
     const gchar * const *sidecar_interfaces;
 
     /**
-     * An implementation of gabble_plugin_create_sidecar().
+     * An implementation of gabble_plugin_create_sidecar_async().
      */
-    GabblePluginCreateSidecarImpl create_sidecar;
+    GabblePluginCreateSidecarImpl create_sidecar_async;
+
+    /**
+     * An implementation of gabble_plugin_create_sidecar_finish().
+     */
+    GabblePluginCreateSidecarFinishImpl create_sidecar_finish;
 
     /**
      * The plugin's version, conventionally a "."-separated sequence of
@@ -93,6 +110,11 @@ struct _GabblePluginInterface {
      * Privacy lists implementing specific statuses
      */
     GabblePluginPrivacyListMap *privacy_list_map;
+
+    /**
+     * An optional callback to create additional channel managers.
+     */
+    GabblePluginCreateChannelManagersImpl create_channel_managers;
 };
 
 GType gabble_plugin_get_type (void);
@@ -108,10 +130,10 @@ gboolean gabble_plugin_implements_sidecar (
     GabblePlugin *plugin,
     const gchar *sidecar_interface);
 
-void gabble_plugin_create_sidecar (
+void gabble_plugin_create_sidecar_async (
     GabblePlugin *plugin,
     const gchar *sidecar_interface,
-    GabbleConnection *connection,
+    GabblePluginConnection *plugin_connection,
     WockySession *session,
     GAsyncReadyCallback callback,
     gpointer user_data);
@@ -132,6 +154,9 @@ const gchar *gabble_plugin_presence_status_for_privacy_list (
     GabblePlugin *plugin,
     const gchar *list_name);
 
+GPtrArray * gabble_plugin_create_channel_managers (GabblePlugin *plugin,
+    GabblePluginConnection *plugin_connection);
+
 /**
  * gabble_plugin_create:
  *
