@@ -17,8 +17,12 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
-#include <wocky/wocky-sasl-utils.h>
+#include <wocky/wocky.h>
 
 typedef struct {
   guint8 *key;
