@@ -17,9 +17,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "wocky-pubsub-node.h"
 #include "wocky-pubsub-node-protected.h"
-#include "wocky-pubsub-node-enumtypes.h"
+#include "wocky-pubsub-node-internal.h"
 
 #include "wocky-namespaces.h"
 #include "wocky-porter.h"
@@ -28,8 +32,8 @@
 #include "wocky-signals-marshal.h"
 #include "wocky-utils.h"
 
-#define DEBUG_FLAG DEBUG_PUBSUB
-#include "wocky-debug.h"
+#define WOCKY_DEBUG_FLAG WOCKY_DEBUG_PUBSUB
+#include "wocky-debug-internal.h"
 
 G_DEFINE_TYPE (WockyPubsubNode, wocky_pubsub_node, G_TYPE_OBJECT)
 
@@ -590,7 +594,7 @@ wocky_pubsub_node_unsubscribe_finish (WockyPubsubNode *self,
     GAsyncResult *result,
     GError **error)
 {
-  wocky_implement_finish_void (self, wocky_pubsub_node_unsubscribe_async)
+  wocky_implement_finish_void (self, wocky_pubsub_node_unsubscribe_async);
 }
 
 WockyStanza *
@@ -626,7 +630,7 @@ wocky_pubsub_node_delete_finish (WockyPubsubNode *self,
     GAsyncResult *result,
     GError **error)
 {
-  wocky_implement_finish_void (self, wocky_pubsub_node_delete_async)
+  wocky_implement_finish_void (self, wocky_pubsub_node_delete_async);
 }
 
 WockyStanza *
@@ -731,7 +735,7 @@ wocky_pubsub_node_list_subscribers_finish (
 {
   wocky_implement_finish_copy_pointer (self,
       wocky_pubsub_node_list_subscribers_async,
-      wocky_pubsub_subscription_list_copy, subscribers)
+      wocky_pubsub_subscription_list_copy, subscribers);
 }
 
 WockyStanza *
@@ -879,7 +883,7 @@ wocky_pubsub_node_list_affiliates_finish (
 {
   wocky_implement_finish_copy_pointer (self,
       wocky_pubsub_node_list_affiliates_async,
-      wocky_pubsub_affiliation_list_copy, affiliates)
+      wocky_pubsub_affiliation_list_copy, affiliates);
 }
 
 /**
@@ -1001,7 +1005,7 @@ wocky_pubsub_node_modify_affiliates_finish (
     GAsyncResult *result,
     GError **error)
 {
-  wocky_implement_finish_void (self, wocky_pubsub_node_modify_affiliates_async)
+  wocky_implement_finish_void (self, wocky_pubsub_node_modify_affiliates_async);
 }
 
 /**
@@ -1103,7 +1107,7 @@ wocky_pubsub_node_get_configuration_finish (
     GError **error)
 {
   wocky_implement_finish_return_copy_pointer (self,
-      wocky_pubsub_node_get_configuration_async, g_object_ref)
+      wocky_pubsub_node_get_configuration_async, g_object_ref);
 }
 
 WockyPorter *
