@@ -17,8 +17,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "wocky-pubsub-service.h"
-#include "wocky-pubsub-service-enumtypes.h"
 #include "wocky-pubsub-service-protected.h"
 
 #include "wocky-porter.h"
@@ -26,11 +29,12 @@
 #include "wocky-pubsub-helpers.h"
 #include "wocky-pubsub-node.h"
 #include "wocky-pubsub-node-protected.h"
+#include "wocky-pubsub-node-internal.h"
 #include "wocky-namespaces.h"
 #include "wocky-signals-marshal.h"
 
-#define DEBUG_FLAG DEBUG_PUBSUB
-#include "wocky-debug.h"
+#define WOCKY_DEBUG_FLAG WOCKY_DEBUG_PUBSUB
+#include "wocky-debug-internal.h"
 
 static gboolean pubsub_service_propagate_event (WockyPorter *porter,
     WockyStanza *event_stanza,
