@@ -27,6 +27,10 @@
  * misc, other
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,8 +45,8 @@
 
 #include "wocky-stanza.h"
 
-#define DEBUG_FLAG DEBUG_XMPP_READER
-#include "wocky-debug.h"
+#define WOCKY_DEBUG_FLAG WOCKY_DEBUG_XMPP_READER
+#include "wocky-debug-internal.h"
 
 /* properties */
 enum {
@@ -686,7 +690,7 @@ wocky_xmpp_reader_push (WockyXmppReader *reader, const guint8 *data,
   g_return_if_fail (priv->state < WOCKY_XMPP_READER_STATE_CLOSED);
 
 #ifdef ENABLE_DEBUG
-  wocky_debug (DEBUG_NET, "Parsing chunk: %.*s", (int)length, data);
+  wocky_debug (WOCKY_DEBUG_NET, "Parsing chunk: %.*s", (int)length, data);
 #endif
 
   parser = priv->parser;
