@@ -34,8 +34,7 @@
 #include <glib/gstdio.h>
 #include <gio/gio.h>
 #include <dbus/dbus-glib.h>
-#include <wocky/wocky-auth-registry.h>
-#include <wocky/wocky-utils.h>
+#include <wocky/wocky.h>
 
 #include <telepathy-glib/channel-iface.h>
 #include <telepathy-glib/dbus.h>
@@ -49,6 +48,7 @@
 
 #define DEBUG_FLAG GABBLE_DEBUG_AUTH
 
+#include <gabble/error.h>
 #include "connection.h"
 #include "debug.h"
 #include "namespaces.h"
