@@ -17,6 +17,12 @@ from jingleshareutils import test_ft_caps_from_contact
 
 import ns
 
+from config import JINGLE_FILE_TRANSFER_ENABLED
+
+if not JINGLE_FILE_TRANSFER_ENABLED:
+    print "NOTE: built with --disable-file-transfer or --disable-voip"
+    raise SystemExit(77)
+
 def test(q, bus, conn, stream):
     client = 'http://telepathy.freedesktop.org/fake-client'
     contact = 'bilbo1@foo.com/Resource'
